use mystem::AppError as mystem_error;
use reqwest::Error as reqwest_error;
use rusqlite::Error as sqlite_error;
use serde_json::Error as serde_error;
use std::{fmt, io::Error as io_error};
use subprocess::PopenError as popen_error;
use telegram_bot::Error as tg_error;

#[derive(Debug)]
pub enum Error {
    UserNotFound,
    SQLITE3Error(sqlite_error),
    TelegramError(tg_error),
    ReqwestError(reqwest_error),
    ConfNotFound,
    WordNotFound,
    WordInStopList,
    IOError(io_error),
    FileNotFound,
    JsonParseError(serde_error),
    PopenError(popen_error),
    MystemError(mystem_error),
    SQLBannedCommand(String),
    SQLInvalidCommand,
    SQLResultTooLong(String),
}

impl fmt::Display for Error {
    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
        write!(f, "An error occurred.")
    }
}

impl From<sqlite_error> for Error {
    fn from(e: sqlite_error) -> Error {
        return Error::SQLITE3Error(e);
    }
}

impl From<tg_error> for Error {
    fn from(e: tg_error) -> Error {
        return Error::TelegramError(e);
    }
}

impl From<reqwest_error> for Error {
    fn from(e: reqwest_error) -> Error {
        return Error::ReqwestError(e);
    }
}

impl From<io_error> for Error {
    fn from(e: io_error) -> Error {
        return Error::IOError(e);
    }
}

impl From<serde_error> for Error {
    fn from(e: serde_error) -> Error {
        return Error::JsonParseError(e);
    }
}

impl From<popen_error> for Error {
    fn from(e: popen_error) -> Error {
        return Error::PopenError(e);
    }
}

impl From<mystem_error> for Error {
    fn from(e: mystem_error) -> Error {
        return Error::MystemError(e);
    }
}
